@@ -1,20 +1,16 @@
 
 from __future__ import annotations
 
-import socket
 import pkg_resources
 from abc import abstractmethod, ABCMeta
-from socket import AddressFamily, SocketKind
-from ssl import SSLSocket, SSLObject
-from typing import Any, Union, Optional, Sequence
+from typing import Optional, Sequence
 from typing_extensions import Final
 
-from .tlv import ProxyProtocolTLV
-from .typing import Address
+from .result import ProxyResult
 
 __all__ = ['__version__', 'ProxyProtocolSyntaxError',
            'ProxyProtocolChecksumError', 'ProxyProtocolIncompleteError',
-           'ProxyProtocolWantRead', 'ProxyProtocolResult', 'ProxyProtocol']
+           'ProxyProtocolWantRead', 'ProxyProtocol']
 
 #: The package version string.
 __version__: str = pkg_resources.require('proxy-protocol')[0].version
@@ -45,7 +41,7 @@ class ProxyProtocolChecksumError(ValueError):
 
     __slots__ = ['result']
 
-    def __init__(self, result: ProxyProtocolResult) -> None:
+    def __init__(self, result: ProxyResult) -> None:
         super().__init__()
         self.result: Final = result
 
@@ -87,60 +83,6 @@ class ProxyProtocolWantRead:
         self.want_line: Final = want_line
 
 
-class ProxyProtocolResult(metaclass=ABCMeta):
-    """Base class for PROXY protocol results."""
-
-    __slots__: Sequence[str] = []
-
-    @property
-    @abstractmethod
-    def proxied(self) -> bool:
-        """True if the result should override the information in the underlying
-        socket.
-
-        """
-        ...
-
-    @property
-    @abstractmethod
-    def source(self) -> Any:
-        """The original source address info for the connection."""
-        ...
-
-    @property
-    @abstractmethod
-    def dest(self) -> Any:
-        """The original destination address info for the connection."""
-        ...
-
-    @property
-    def family(self) -> AddressFamily:
-        """The original socket address family."""
-        return socket.AF_UNSPEC
-
-    @property
-    def protocol(self) -> Optional[SocketKind]:
-        """The original socket protocol."""
-        return None
-
-    @property
-    @abstractmethod
-    def tlv(self) -> ProxyProtocolTLV:
-        """Additional information about the connection."""
-        ...
-
-    @property
-    def _sockname(self) -> Address:
-        return None
-
-    @property
-    def _peername(self) -> Address:
-        return None
-
-    def __str__(self) -> str:
-        return f'{self.__class__.__name__!s}()'
-
-
 class ProxyProtocol(metaclass=ABCMeta):
     """The base class for PROXY protocol implementations."""
 
@@ -158,7 +100,7 @@ class ProxyProtocol(metaclass=ABCMeta):
         ...
 
     @abstractmethod
-    def parse(self, data: bytes) -> ProxyProtocolResult:
+    def unpack(self, data: bytes) -> ProxyResult:
         """Parse a PROXY protocol header from the given bytestring and return
         information about the original connection.
 
@@ -178,25 +120,13 @@ class ProxyProtocol(metaclass=ABCMeta):
         ...
 
     @abstractmethod
-    def build(self, source: Address, dest: Address, *, family: AddressFamily,
-              protocol: Optional[SocketKind] = None,
-              ssl: Union[None, SSLObject, SSLSocket] = None,
-              unique_id: Optional[bytes] = None,
-              proxied: bool = True,
-              dnsbl: Optional[str] = None) -> bytes:
-        """Builds a PROXY protocol v1 header that may be sent at the beginning
-        of an outbound, client-side connection to indicate the original
+    def pack(self, result: ProxyResult) -> bytes:
+        """Builds a PROXY protocol header that may be sent at the beginning of
+        an outbound, client-side connection to indicate the original
         information about the connection.
 
         Args:
-            source: The original source address of the connection.
-            dest: The original destination address of the connection.
-            family: The original socket family.
-            protocol: The original socket protocol.
-            ssl: The original socket SSL information.
-            unique_id: The original connection unique identifier.
-            proxied: True if the connection should be considered proxied.
-            dnsbl: The DNSBL lookup result, if any.
+            result: The PROXY protocol result to build into a header.
 
         Raises:
             :exc:`KeyError`: This PROXY protocol header format does not support
