@@ -4,11 +4,10 @@ from __future__ import annotations
 import logging
 from abc import abstractmethod, ABCMeta
 from asyncio import Task, AbstractEventLoop
-from asyncio.protocols import BufferedProtocol, BaseProtocol
+from asyncio.protocols import BufferedProtocol
 from asyncio.transports import Transport, BaseTransport
 from collections import deque
 from functools import partial
-from socket import AddressFamily, SocketKind
 from typing import Type, Optional, Tuple, Deque
 from typing_extensions import Final
 from uuid import uuid4
@@ -16,13 +15,15 @@ from uuid import uuid4
 from . import Address
 from .. import ProxyProtocol
 from ..dnsbl import Dnsbl
-from ..sock import SocketInfo
+from ..build import build_transport_result
+from ..result import ProxyResult
+from ..sock import SocketInfo, SocketInfoLocal
 
 __all__ = ['DownstreamProtocol', 'UpstreamProtocol']
 
 _log = logging.getLogger(__name__)
 
-_Connect = Tuple[BaseTransport, BaseProtocol]
+_Connect = Tuple[BaseTransport, 'UpstreamProtocol']
 
 
 class _Base(BufferedProtocol, metaclass=ABCMeta):
@@ -45,6 +46,16 @@ class _Base(BufferedProtocol, metaclass=ABCMeta):
         assert sock_info is not None
         return sock_info
 
+    @property
+    def transport(self) -> Transport:
+        transport = self._transport
+        assert transport is not None
+        return transport
+
+    @property
+    def connected(self) -> bool:
+        return self._transport is not None
+
     def close(self) -> None:
         if self._transport is not None:
             self._transport.close()
@@ -70,7 +81,7 @@ class _Base(BufferedProtocol, metaclass=ABCMeta):
     def connection_made(self, transport: BaseTransport) -> None:
         assert isinstance(transport, Transport)
         self._transport = transport
-        self._sock_info = SocketInfo(transport, unique_id=uuid4().bytes)
+        self._sock_info = SocketInfoLocal(transport, unique_id=uuid4().bytes)
 
     def connection_lost(self, exc: Optional[Exception]) -> None:
         self.close()
@@ -114,7 +125,8 @@ class DownstreamProtocol(_Base):
         self._upstream_factory = partial(upstream_protocol, self, buf_len,
                                          upstream.pp)
 
-    def _set_client(self, connect_task: Task[_Connect]) -> None:
+    def _set_client(self, result: ProxyResult,
+                    connect_task: Task[_Connect]) -> None:
         dnsbl_task = self._dnsbl_task
         assert dnsbl_task is not None
         try:
@@ -125,13 +137,13 @@ class DownstreamProtocol(_Base):
             _log.exception('[%s] Connection failed: %s',
                            self.id.hex(), self.upstream)
         else:
-            callback = partial(self._send_initial, upstream)
+            callback = partial(self._send_initial, upstream, result)
             dnsbl_task.add_done_callback(callback)
 
-    def _send_initial(self, upstream: UpstreamProtocol,
+    def _send_initial(self, upstream: UpstreamProtocol, result: ProxyResult,
                       dnsbl_task: Task[Optional[str]]) -> None:
         self._upstream = upstream
-        upstream.write_header(dnsbl_task.result())
+        upstream.write_header(dnsbl_task.result(), result)
         waiting = self._waiting
         while waiting:
             data = waiting.popleft()
@@ -149,7 +161,8 @@ class DownstreamProtocol(_Base):
             loop.create_connection(self._upstream_factory,
                                    self.upstream.host, self.upstream.port or 0,
                                    ssl=self.upstream.ssl))
-        connect_task.add_done_callback(self._set_client)
+        result = build_transport_result(transport, unique_id=self.id)
+        connect_task.add_done_callback(partial(self._set_client, result))
 
     def connection_lost(self, exc: Optional[Exception]) -> None:
         super().connection_lost(exc)
@@ -181,21 +194,16 @@ class UpstreamProtocol(_Base):
         super().close()
         self.downstream.close()
 
-    def build_pp_header(self, dnsbl: Optional[str]) -> bytes:
-        sock_info = self.downstream.sock_info
-        sock = sock_info.socket
-        ssl_object = sock_info.transport.get_extra_info('ssl_object')
-        try:
-            protocol: Optional[SocketKind] = SocketKind(sock.type)
-        except ValueError:
-            protocol = None
-        return self.pp.build(sock_info.peername, sock_info.sockname,
-                             family=AddressFamily(sock.family),
-                             protocol=protocol, unique_id=self.downstream.id,
-                             ssl=ssl_object, dnsbl=dnsbl)
+    def build_pp_header(self, dnsbl: Optional[str],
+                        result: ProxyResult) -> bytes:
+        if self.downstream.connected:
+            result = build_transport_result(self.downstream.transport,
+                                            unique_id=self.downstream.id,
+                                            dnsbl=dnsbl)
+        return self.pp.pack(result)
 
-    def write_header(self, dnsbl: Optional[str]) -> None:
-        header = self.build_pp_header(dnsbl)
+    def write_header(self, dnsbl: Optional[str], result: ProxyResult) -> None:
+        header = self.build_pp_header(dnsbl, result)
         self.write(header)
 
     def proxy_data(self, data: bytes) -> None:
