@@ -4,18 +4,15 @@ from __future__ import annotations
 import socket
 from ipaddress import IPv4Address, IPv6Address
 from socket import AddressFamily, SocketKind
-from ssl import SSLSocket, SSLObject
 from struct import Struct
-from typing import cast, Union, Optional, Tuple, Sequence
+from typing import Optional, Sequence
 from typing_extensions import Final
 
-from . import ProxyProtocolWantRead, ProxyProtocolResult, ProxyProtocol, \
-    ProxyProtocolSyntaxError, ProxyProtocolChecksumError, \
-    ProxyProtocolIncompleteError
-from .result import ProxyProtocolResultLocal, ProxyProtocolResultUnknown, \
-    ProxyProtocolResultIPv4, ProxyProtocolResultIPv6, ProxyProtocolResultUnix
-from .tlv import ProxyProtocolTLV, ProxyProtocolSSLTLV, ProxyProtocolExtTLV
-from .typing import Address, PeerCert
+from . import ProxyProtocolWantRead, ProxyProtocol, ProxyProtocolSyntaxError, \
+    ProxyProtocolChecksumError, ProxyProtocolIncompleteError
+from .result import is_ipv4, is_ipv6, is_unix, ProxyResult, ProxyResultLocal, \
+    ProxyResultUnknown, ProxyResultIPv4, ProxyResultIPv6, ProxyResultUnix
+from .tlv import ProxyProtocolTLV
 
 __all__ = ['ProxyProtocolV2Header', 'ProxyProtocolV2']
 
@@ -67,18 +64,18 @@ class ProxyProtocolV2(ProxyProtocol):
     def is_valid(self, signature: bytes) -> bool:
         return signature[0:8] == b'\r\n\r\n\x00\r\nQ'
 
-    def parse(self, data: bytes) -> ProxyProtocolResult:
+    def unpack(self, data: bytes) -> ProxyResult:
         if len(data) < 16:
             want_read = ProxyProtocolWantRead(16 - len(data))
             raise ProxyProtocolIncompleteError(want_read)
         header_data, data = data[0:16], data[16:]
-        header = self.parse_header(header_data)
+        header = self.unpack_header(header_data)
         if len(data) < header.data_len:
             want_read = ProxyProtocolWantRead(header.data_len - len(data))
             raise ProxyProtocolIncompleteError(want_read)
-        return self.parse_data(header, header_data, data)
+        return self.unpack_data(header, header_data, data)
 
-    def parse_header(self, header_data: bytes) -> ProxyProtocolV2Header:
+    def unpack_header(self, header_data: bytes) -> ProxyProtocolV2Header:
         """Parse the PROXY protocol v2 header.
 
         Args:
@@ -98,9 +95,9 @@ class ProxyProtocolV2(ProxyProtocol):
         return ProxyProtocolV2Header(command=command, family=family,
                                      protocol=protocol, data_len=data_len)
 
-    def parse_data(self, header: ProxyProtocolV2Header,
-                   header_data: bytes, data: bytes) \
-            -> ProxyProtocolResult:
+    def unpack_data(self, header: ProxyProtocolV2Header,
+                    header_data: bytes, data: bytes) \
+            -> ProxyResult:
         """Parse the address information read from the stream after the v2
         header.
 
@@ -115,11 +112,11 @@ class ProxyProtocolV2(ProxyProtocol):
         """
         if header.command not in ('local', 'proxy'):
             raise ProxyProtocolSyntaxError('Invalid proxy protocol command')
-        result: ProxyProtocolResult
+        result: ProxyResult
         if header.command == 'local':
             addr_data, tlv_data = b'', data
             tlv = ProxyProtocolTLV(tlv_data)
-            result = ProxyProtocolResultLocal(tlv)
+            result = ProxyResultLocal(tlv=tlv)
         elif header.family == socket.AF_INET:
             addr_len = self._ipv4_format.size
             addr_data, tlv_data = data[:addr_len], data[addr_len:]
@@ -128,8 +125,8 @@ class ProxyProtocolV2(ProxyProtocol):
             source_addr4 = (IPv4Address(source_ip), source_port)
             dest_addr4 = (IPv4Address(dest_ip), dest_port)
             tlv = ProxyProtocolTLV(tlv_data)
-            result = ProxyProtocolResultIPv4(source_addr4, dest_addr4,
-                                             protocol=header.protocol, tlv=tlv)
+            result = ProxyResultIPv4(source_addr4, dest_addr4,
+                                     protocol=header.protocol, tlv=tlv)
         elif header.family == socket.AF_INET6:
             addr_len = self._ipv6_format.size
             addr_data, tlv_data = data[:addr_len], data[addr_len:]
@@ -138,8 +135,8 @@ class ProxyProtocolV2(ProxyProtocol):
             source_addr6 = (IPv6Address(source_ip), source_port)
             dest_addr6 = (IPv6Address(dest_ip), dest_port)
             tlv = ProxyProtocolTLV(tlv_data)
-            result = ProxyProtocolResultIPv6(source_addr6, dest_addr6,
-                                             protocol=header.protocol, tlv=tlv)
+            result = ProxyResultIPv6(source_addr6, dest_addr6,
+                                     protocol=header.protocol, tlv=tlv)
         elif header.family == socket.AF_UNIX:
             addr_len = self._unix_format.size
             addr_data, tlv_data = data[:addr_len], data[addr_len:]
@@ -147,109 +144,50 @@ class ProxyProtocolV2(ProxyProtocol):
             source_addru = source_addr_b.rstrip(b'\x00').decode('ascii')
             dest_addru = dest_addr_b.rstrip(b'\x00').decode('ascii')
             tlv = ProxyProtocolTLV(tlv_data)
-            result = ProxyProtocolResultUnix(source_addru, dest_addru,
-                                             protocol=header.protocol, tlv=tlv)
+            result = ProxyResultUnix(source_addru, dest_addru,
+                                     protocol=header.protocol, tlv=tlv)
         else:
-            return ProxyProtocolResultUnknown()
+            return ProxyResultUnknown()
         if not tlv.verify_checksum(header_data, addr_data):
             raise ProxyProtocolChecksumError(result)
         return result
 
-    def build(self, source: Address, dest: Address, *, family: AddressFamily,
-              protocol: Optional[SocketKind] = None,
-              ssl: Union[None, SSLSocket, SSLObject] = None,
-              unique_id: Optional[bytes] = None,
-              proxied: bool = True,
-              dnsbl: Optional[str] = None) -> bytes:
-        addresses = self.build_addresses(source, dest, family=family)
-        tlv = self.build_tlv(ssl, unique_id, dnsbl)
+    def pack(self, result: ProxyResult) -> bytes:
+        addresses = self._pack_addresses(result)
+        tlv = ProxyProtocolTLV(init=result.tlv, crc32c=None, auto_crc32c=True)
         data_len = len(addresses) + tlv.size
-        header = self.build_header(data_len, family=family, protocol=protocol,
-                                   proxied=proxied)
+        header = self._pack_header(data_len, result)
         tlv = tlv.with_checksum(header, addresses)
         return header + addresses + bytes(tlv)
 
-    def build_header(self, data_len: int, *,
-                     family: AddressFamily,
-                     protocol: Optional[SocketKind] = None,
-                     proxied: bool = True) -> bytes:
-        """Builds the 16-byte block that begins every PROXY protocol v2 header.
-
-        Args:
-            data_len: The length of the data (addresses + TLV).
-            family: The original socket family.
-            protocol: The original socket protocol.
-            proxied: True if the connection should be considered proxied.
-
-        """
-        byte_12 = 0x20 + self._commands_r['proxy' if proxied else 'local']
+    def _pack_header(self, data_len: int, result: ProxyResult) \
+            -> bytes:
+        command = 'proxy' if result.proxied else 'local'
+        family = result.family
+        protocol = result.protocol
+        byte_12 = 0x20 + self._commands_r[command]
         byte_13 = self._families_r[family] + self._protocols_r[protocol]
         return b'\r\n\r\n\x00\r\nQUIT\n%b' % \
             self._header_format.pack(byte_12, byte_13, data_len)
 
-    def build_addresses(self, source: Address, dest: Address, *,
-                        family: AddressFamily) -> bytes:
-        """Builds the block of address data is contained in a PROXY protocol v2
-        header.
-
-        Args:
-            source: The original source address of the connection.
-            dest: The original destination address of the connection.
-            family: The original socket family.
-
-        """
-        if family == socket.AF_INET:
-            source = cast(Tuple[str, int], source)
-            dest = cast(Tuple[str, int], dest)
-            source_ip = IPv4Address(source[0]).packed
-            source_port = source[1]
-            dest_ip = IPv4Address(dest[0]).packed
-            dest_port = dest[1]
+    def _pack_addresses(self, result: ProxyResult) -> bytes:
+        if is_ipv4(result):
+            source_ip = result.source[0].packed
+            source_port = result.source[1]
+            dest_ip = result.dest[0].packed
+            dest_port = result.dest[1]
             return self._ipv4_format.pack(source_ip, dest_ip,
                                           source_port, dest_port)
-        elif family == socket.AF_INET6:
-            source = cast(Tuple[str, int, int, int], source)
-            dest = cast(Tuple[str, int, int, int], dest)
-            source_ip = IPv6Address(source[0]).packed
-            source_port = source[1]
-            dest_ip = IPv6Address(dest[0]).packed
-            dest_port = dest[1]
+        elif is_ipv6(result):
+            source_ip = result.source[0].packed
+            source_port = result.source[1]
+            dest_ip = result.dest[0].packed
+            dest_port = result.dest[1]
             return self._ipv6_format.pack(source_ip, dest_ip,
                                           source_port, dest_port)
-        elif family == socket.AF_UNIX:
-            source = cast(str, source)
-            dest = cast(str, dest)
-            source_b = source.encode('ascii')
-            dest_b = dest.encode('ascii')
+        elif is_unix(result):
+            source_b = result.source.encode('ascii')
+            dest_b = result.dest.encode('ascii')
             return self._unix_format.pack(source_b, dest_b)
         else:
             return b''
-
-    def build_tlv(self, ssl: Union[None, SSLSocket, SSLObject],
-                  unique_id: Optional[bytes], dnsbl: Optional[str]) \
-            -> ProxyProtocolTLV:
-        """Builds the TLV data written after the PROXY protocol v2 address
-        data.
-
-        Args:
-            ssl: The SSL information for the connection.
-            unique_id: The unique ID of the connection.
-            dnsbl: The DNSBL lookup result, if any.
-
-        """
-        ssl_tlv: Optional[ProxyProtocolSSLTLV] = None
-        ext_tlv: Optional[ProxyProtocolExtTLV] = None
-        if dnsbl is not None:
-            ext_tlv = ProxyProtocolExtTLV(init=ext_tlv, dnsbl=dnsbl)
-        if ssl is not None:
-            cipher, version, secret_bits = ssl.cipher() or (None, None, None)
-            peercert: Optional[PeerCert] = ssl.getpeercert()
-            ssl_tlv = ProxyProtocolSSLTLV(
-                has_ssl=True, verify=True,
-                has_cert_conn=(peercert is not None),
-                cipher=cipher, version=version)
-            ext_tlv = ProxyProtocolExtTLV(
-                init=ext_tlv, compression=ssl.compression(),
-                secret_bits=secret_bits, peercert=peercert, dnsbl=dnsbl)
-        return ProxyProtocolTLV(unique_id=unique_id, ssl=ssl_tlv, ext=ext_tlv,
-                                auto_crc32c=True)
